#!/usr/bin/env python
# -*- coding: utf-8 -*-
# dialoge.py
##© Claus Albowski
## Quelle der Vorlagendatei: effbot.org   tkinterbook
## Insbesondere das Layout ist geaendert.
#### Hinweis: Fuer tkFileDialog /tkinter.filedialog muss das jweilige
#### Modul mit diesem Namen importiert werden!

import platform
versionsNummer=platform.python_version()[0]
if versionsNummer=='3': from tkinter import *
else: from Tkinter import *

#### Dialog-Vorlage --------------------------------------------------
class Dialog(Toplevel, object):

    fg,bg='black','white'
    @staticmethod
    def Setze_FG_BG(fg,bg):
        Dialog.fg=fg
        Dialog.bg=bg    

    def __init__(self, parent, title = None):

        Toplevel.__init__(self, parent)
        self.transient(parent)
        if title: self.title(title)
        self.parent = parent
        self.result = None

        self.bodyFrame = Frame(self)
        self.initial_focus = self.bodyFrame
        self.body()
        self.bodyFrame.grid(row=0,column=0)
        self.buttonFrame = Frame(self)
        self.buttons()
        self.buttonFrame.grid(row=1,column=0)
        # self.grab_set()
        if not self.initial_focus:
            self.initial_focus = self
        self.protocol("WM_DELETE_WINDOW", self.cancel)
        self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                  parent.winfo_rooty()+50))
        self.initial_focus.focus_set()
        self.wait_window(self)

    # construction hooks

    def body(self):
        '''erzeugt den speziellen Inhalt des Dialogs.
        Muss ueberschrieben werden.'''
        pass

    def buttons(self):
        '''Standard Button-Abschnitt.
        ggf. ueberschreiben'''
        
        ja = Button(self.buttonFrame, text="OK", width=10, command=self.ok, default=ACTIVE)
        ja.grid(row=0,column=0, padx=5, pady=5)
        abbruch = Button(self.buttonFrame, text="Cancel", width=10, command=self.cancel)
        abbruch.grid(row=0,column=1, padx=5, pady=5)
        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)


    # standard button semantics

    def ok(self, event=None):
        '''Ereignisbehandlung fuer den ok-Button'''
        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return
        self.withdraw()
        self.update_idletasks()

        self.anwenden()
        self.beenden()

    def cancel(self, event=None):
        '''Ereignisbehandlung fuer den ok-Button.
        Methode ueberschreiben, wenn noch Aktionen auszufuehren sind.'''
        result=None
        self.beenden()

    def beenden(self):
        '''put focus back to the parent window'''
        self.parent.focus_set()
        self.destroy()
    # command hooks

    def validate(self):
        '''Methode zum Ueberpruefen; ggf. ueberschreiben.'''
        return 1 # override

    def anwenden(self):
        '''definiert, was im konkreten Fall zu tun ist'''
        pass
        # override !

#### Bestaetigungs-Dialog --------------------------------------------------
class BestaetigungsDialog(Dialog):

    def __init__(self, parent, title = None, ausgabe=''):
        self.__ausgabe=ausgabe
        Dialog.__init__(self, parent, title)

    #### leere Methoden ueberschreiben
    def body(self):
        self.ausgabe_Label = Label(self.bodyFrame, text=self.__ausgabe)
        #self.ausgabe_Label.grid(row=0,column=0,columnspan=2)
        self.ausgabe_Label.pack()

    def buttons(self):
        '''Standard Button-Abschnitt ueberschreiben'''
        
        ja = Button(self.buttonFrame, text="ja", width=10, command=self.ok)
        ja.grid(row=0,column=0, padx=5, pady=5)
        nein = Button(self.buttonFrame, text="nein", width=10, command=self.not_ok)
        nein.grid(row=0,column=1, padx=5, pady=5)
        abbruch = Button(self.buttonFrame, text="Cancel", width=10, command=self.cancel, default=ACTIVE)
        abbruch.grid(row=0,column=2, padx=5, pady=5)
        self.bind("<Return>", self.cancel)
        self.bind("<Escape>", self.cancel)
        
    def ok(self, event=None):
        '''Ereignisbehandlung fuer den ja-Button'''
        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return
        self.withdraw()
        self.update_idletasks()

        self.anwenden(True)
        self.beenden()
        
    def not_ok(self, event=None):
        '''Ereignisbehandlung fuer den nein-Button'''
        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return
        self.withdraw()
        self.update_idletasks()

        self.anwenden(False)
        self.beenden()
        
    def anwenden(self, value=None):
        self.result=value

#### Eingabe-Dialog --------------------------------------------------
class EingabeDialog(Dialog):

    def __init__(self, parent, title = None, aufgabe='', wert=''):
        '''Der Wert muss auch fuer Zahlen als String uebergeben werden.'''
        self.aufgabe=aufgabe
        self.wert=wert
        Dialog.__init__(self, parent, title)

    def body(self):
        self.aufgabe_Label = Label(self.bodyFrame,
                                   text=self.aufgabe)
        self.aufgabe_Label.grid(row=0,column=0)
        self.EingabeVariable = StringVar()
        self.EingabeVariable.set(self.wert)
        self.Eingabe = Entry(self.bodyFrame,
                             textvariable=self.EingabeVariable,
                             fg=Dialog.fg,bg=Dialog.bg)
        self.Eingabe.grid(row=1,column=0)
        self.Eingabe.focus_set()

    def anwenden(self):
        self.result = self.EingabeVariable.get()

    def cancel(self, event=None):
        '''Methode ueberschrieben, um den Inhalt zu loeschen.'''
        self.EingabeVariable.set('')
        super(EingabeDialog,self).cancel()


#### Mehrfach-Eingaben-Dialog --------------------------------------------------
class N_EingabenDialog(Dialog):

    def __init__(self, parent, title = None, namenZu=[], gefragteWerte=[]):
        self.namenZu=namenZu
        if len(gefragteWerte)==0: gefragteWerte=['']*len(namenZu)
        self.gefragt=gefragteWerte
        Dialog.__init__(self, parent, title)

    #### leere Methoden ueberschreiben
    def body(self):
        assert len(self.namenZu)==len(self.gefragt), "Listen verschieden lang!"
        labelListe,self.variablenListe,self.eingabenListe=[],[],[]
        for i in range(len(self.namenZu)):
            labelListe.append(Label(self.bodyFrame,
                                    text=self.namenZu[i]))
            labelListe[i].grid(row=i,column=0)            
            self.variablenListe.append(StringVar())
            self.variablenListe[i].set(self.gefragt[i])
            self.eingabenListe.append(Entry(self.bodyFrame,
                                            textvariable=self.variablenListe[i],
                                            fg=Dialog.fg,bg=Dialog.bg))
            self.eingabenListe[i].grid(row=i,column=1)
            self.ersteEingabe=self.eingabenListe[0]
            self.ersteEingabe.focus_set()
    
    def buttonbox(self):
        '''add standard button box.
        override if you don't want the standard buttons'''
        pass

    def anwenden(self):
        self.result = []
        for variable in self.eingabenListe:
            self.result.append(variable.get())

    def cancel(self, event=None):
        '''Methode ueberschrieben, um den Inhalt zu loeschen.'''
        for var in self.variablenListe: var.set('')
        super(N_EingabenDialog,self).cancel()

#### Auswahl-Dialog --------------------------------------------------
class AuswahlDialog(Dialog):

    def __init__(self, parent, title, auswahlen, einzeln=False):
        '''Als Parameter muss eine Liste von Strings uebergeben werden.
        Wird nur eine einzige Auswahl akzeptiert, wird diese zurueckgegeben,
        sonst eine Liste der Ergebnisse.'''
        self.parent=parent
        self.einzeln=einzeln
        self.listboxVariable = StringVar()
        self.auswahlen=auswahlen
        self.ausgewaehlt=[]
        Dialog.__init__(self, parent, title)

    #### leere Methoden ueberschreiben
    def body(self):
        if self.einzeln: sel=SINGLE
        else: sel=EXTENDED
        self.listbox = Listbox(self.bodyFrame,
                               width=30, height=10,
                               listvariable=self.listboxVariable,
                               selectmode=sel,
                               fg=Dialog.fg,bg=Dialog.bg)
        for item in self.auswahlen:
            self.listbox.insert(END, item)
        self.listbox.grid(row=0,column=0)
        self.scrollY=Scrollbar(self.bodyFrame,
                               orient=VERTICAL,
                               width=16,
                               command=self.listbox.yview)
        self.scrollY.grid(row=0,column=1, sticky=N+S)
        self.listbox['yscrollcommand'] = self.scrollY.set
        self.listbox.bind("<Double-Button-1>", self.ok)

    def anwenden(self):
        '''Sehr eigenartig! Allerdings: So geht es.'''
        items=[]
        try:
            items = map(int, self.listbox.curselection())
            ## Versionsunabhaengigkeit von map herstellen:
            if versionsNummer=='3': items = list(items)
        except Exception: pass
        for item in items:
            self.ausgewaehlt+=[self.listbox.get(item)]
        if self.ausgewaehlt==[]:
            self.cancel()
        elif self.einzeln: self.result=self.ausgewaehlt[0]
        else: self.result=self.ausgewaehlt

#### Meldung-Dialog --------------------------------------------------
class MeldungDialog(Dialog):

    def __init__(self, parent, title = None, ausgabe=''):
        self.__ausgabe=ausgabe
        Dialog.__init__(self, parent, title)

    #### leere Methoden ueberschreiben
    def body(self):
        self.ausgabe_Label = Label(self.bodyFrame, text=self.__ausgabe)
        #self.ausgabe_Label.grid(row=0,column=0,columnspan=2)
        self.ausgabe_Label.pack()

    def anwenden(self):
        self.result='ok'


#### TextAusgabe-Dialog ----------------------------------------------
class TextAusgabeDialog(Dialog):
    '''Ausgabe von Text in einem Textfeld'''
    def __init__(self, parent, title = None, ueberschrift='', zeilen=[]):
        '''Der Wert muss auch fuer Zahlen als String uebergeben werden.'''
        self.ueberschrift=ueberschrift
        self.zeilen=zeilen
        Dialog.__init__(self, parent, title)

    #### leere Methoden ueberschreiben
    def body(self):
        '''Stellt den Text dar.'''
        self.text = Text(self.bodyFrame, fg=Dialog.fg,bg=Dialog.bg)
        self.text.insert(END, self.ueberschrift+'\n','ueber')
        self.text.tag_config('ueber',font=('Arial', '16'),justify=CENTER)
        for zeile in self.zeilen:
            self.text.insert(END, str(zeile)+'\n','normal')
        self.text.tag_config('normal',font=('Arial', '12'),justify=LEFT)
        self.text.config(state=DISABLED)
        self.text.grid(row=0,column=0)
        self.scrollX=Scrollbar(self.bodyFrame,
                               orient=HORIZONTAL,
                               command=self.text.xview)
        self.scrollY=Scrollbar(self.bodyFrame,
                               orient=VERTICAL,
                               command=self.text.yview)
        self.scrollX.grid(row=1,column=0, sticky=W+E)
        self.scrollY.grid(row=0,column=1, sticky=N+S)
        self.text['xscrollcommand'] = self.scrollX.set
        self.text['yscrollcommand'] = self.scrollY.set

    def buttons(self):
        '''ueberschreibt den Standard Button-Abschnitt.'''        
        ja = Button(self.buttonFrame, text=u'schließen', width=10, command=self.ok, default=ACTIVE)
        ja.pack()
        self.bind("<Return>", self.ok)
        
#### TabellenAusgabe-Dialog ----------------------------------------------
class TabellenAusgabeDialog(Dialog):
    '''Die Groesse der dargestellten Tabelle ist auf 30 Zeilen begrenzt.'''
##quelle des Originals:
##https://stackoverflow.com/questions/9348264/does-tkinter-have-a-table-widget
## to do: quelle pruefen
##https://github.com/dmnfarrell/tkintertable/tree/master/tkintertable
    
    def laengenFehler(self,werte):
        '''testet, ob alle Teillisten gleich lang sind'''
        laenge=len(werte[0])
        for i in range(1, len(werte)):
            if len(werte[i])!=laenge: return True
        return False
    
    def __init__(self, parent, title = "Tabelle", werte=[[""]], ueberschrift='Tabelle'):
        '''die werte muessen als Liste von Listen uebergeben werden.
        Die Werte werden zu Strings in Stringvariablen umgewandelt.'''
        self.ueberschrift=ueberschrift
        if len(werte)==0:
            MeldungDialog(self, "Fehler", "keine Daten!")
            self.werte=[[""]]
        elif self.laengenFehler(werte):
            MeldungDialog(self, "Fehler", "Daten verschieden lang!")
            self.werte=[[""]]
        else:
            self.werte=[]
            for i in range(len(werte)):
                ## hier muss ein Bezug zu root mit hineinkommen
                zeile=[]
                for j in range(len(werte[0])):
                    zeile.append(StringVar())
                self.werte.append(zeile)
                for j in range(len(werte[0])):
                    self.werte[i][j].set(str(werte[i][j]))
        Dialog.__init__(self, parent, title)

    #### leere Methoden ueberschreiben
    def body(self):
        '''Stellt eine Tabelle der ausgewaehlten Groessen dar.'''
        spalten,zeilen=len(self.werte[0]),len(self.werte)
        for i in range(zeilen): #Rows
            for j in range(spalten): #Columns
                b = Entry(self.bodyFrame,
                          textvariable=self.werte[i][j],
                          justify=CENTER,
                          fg=Dialog.fg,bg=Dialog.bg)
                b.grid(row=i, column=j)

    def buttons(self):
        '''ueberschreibt den Standard Button-Abschnitt.'''        
        ja = Button(self.buttonFrame, text=u'schließen', width=10, command=self.ok, default=ACTIVE)
        ja.grid(row=len(self.werte[0])+1, column=0)
        self.bind("<Return>", self.ok)
        
#### Zum Testen -------------------------------------------------------
class Test_Application(Frame):
    
    def __init__(self, master=None):
        Frame.__init__(self, master)
        b1=Button(self, text="Meldung", width=15, command=self.MeldungOeffnen)
        b1.grid(row=0,column=0)
        b2=Button(self, text="Eingabe", width=15, command=self.EingabeDialogOeffnen)
        b2.grid(row=0,column=1)
        b3=Button(self, text="Auswahl", width=15, command=self.AuswahlDialogOeffnen)
        b3.grid(row=0,column=2)
        b4=Button(self, text="Nur einen", width=15, command=self.EinzelwahlDialogOeffnen)
        b4.grid(row=0,column=3)
        b5=Button(self, text="Zwei Werte", width=15, command=self.N_EingabeDialogOeffnen)
        b5.grid(row=1,column=0)
        b6=Button(self, text="Textausgabe", width=15, command=self.TextAusgabeOeffnen)
        b6.grid(row=1,column=1)
        b7=Button(self, text=u"TabellenAusgabe",width=15, command=self.TabellenAusgabeOeffnen)
        b7.grid(row=1,column=2)
        b8=Button(self, text=u"ja / nein",width=15, command=self.BestaetigenOeffnen)
        b8.grid(row=1,column=3)
        b9=Button(self, text=u"Schließen",width=15, command=self.Schliessen)
        b9.grid(row=1,column=4)
        self.pack()

        self.__ergebnis=[]

    def EingabeDialogOeffnen(self):
        dialog=EingabeDialog(self, 'Abfrage', 'Text eingeben')
        MeldungDialog(self,'Eingabe war ',dialog.result)

    def N_EingabeDialogOeffnen(self):
        dialog=N_EingabenDialog(self,
                                'Abfragen',
                                ['Bestand','Anfangswert'],
                                ['x',str(20.3)])
        MeldungDialog(self,'Eingaben waren ',str(dialog.result))
        
    def AuswahlDialogOeffnen(self):
        dialog=AuswahlDialog(self, 'Auswahl', ['Hund', 'Katze', 'Maus'])
        MeldungDialog(self,'Ergebnis ',str(dialog.result))
        
    def EinzelwahlDialogOeffnen(self):
        auswahlen=['A', 'B', 'C','1','2','3','4','5','6','7','8','9','10','11']
        dialog=AuswahlDialog(self, 'Auswahl', auswahlen, True)
        MeldungDialog(self,'Ergebnis ',str(dialog.result))
        
    def MeldungOeffnen(self):
        dialog=MeldungDialog(self, 'Meldung', u'wirklich schließen? ')
        MeldungDialog(self,'Ergebnis ',dialog.result)
        
    def TextAusgabeOeffnen(self):
        dialog=TextAusgabeDialog(self,  u'Textausgabe', 'Tiere', ['Hund','und','Katze'])
        MeldungDialog(self,'Kein Ergebnis ')
        
    def TabellenAusgabeOeffnen(self):
        dialog=TabellenAusgabeDialog(self,  u'Tabellenausgabe', werte=[[1,2],[4,5]])
        MeldungDialog(self,'Kein Ergebnis ')

    def BestaetigenOeffnen(self):
        dialog=BestaetigungsDialog(self, u'Prüfen', 'Wollen Sie das wirklich?')
        if dialog.result==None:
            MeldungDialog(self,'Ergebnis :','abgebrochen')
        elif dialog.result:
            MeldungDialog(self,'Ergebnis :','Ja angeklickt!')
        else:
            MeldungDialog(self,'Ergebnis :','Nein angeklickt!')
        
    def Schliessen(self):
        self.quit()

## Tests:
##root=Tk()
##app=Frame(root)
##dialog=TabellenAusgabeDialog(app, '?','T')
##print dialog.anwenden() nicht direkt aufurfen!!
##print dialog.result
##app.mainloop()

##root=Tk()
##app=Frame(root)
##dialog=TabellenAusgabeDialog(app, werte=[[1,2],[4,5]])
##app.mainloop()
##root.destroy()

#### als Hauptfenster starten
if __name__=='__main__':
    root = Tk()
    app = Test_Application(master=root)
    app.master.title('Hauptfenster')
    app.mainloop()
    try:
        root.destroy()
    except Exception: pass

